/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import java.io.IOException;
import types.Complexe;
import types.ErreurCalculException;
import types.Operateur;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CIntegraleDansFormule extends CCalculBase {
	public CCalculBase calculASommer;
	public CCalculBase bornea;
	public CCalculBase borneb;
	public String nomVariable;
	private boolean dansFormuleComplexe;
	
	/**
	 * Constructeur vide pour serialization
	 */
	public CIntegraleDansFormule() {
	}
	
	public CIntegraleDansFormule(CListeObjets liste, CCalculBase calculASommerInit, CCalculBase indiceDebutInit,
			CCalculBase indiceFinInit, String nomVariableInit, boolean dansFormuleComplexeInit) {
		super(liste);
		calculASommer = calculASommerInit;
		bornea = indiceDebutInit;
		borneb = indiceFinInit;
		nomVariable = nomVariableInit;
		dansFormuleComplexe = dansFormuleComplexeInit;
	}

  @Override
  protected long nature() {
  	return natIntegraleDansFormule;
  }  
	
	@Override
	public boolean dependDeVariable(int ind) {
		return calculASommer.dependDeVariable(ind) || bornea.dependDeVariable(ind)
			|| borneb.dependDeVariable(ind);
	}

	@Override
	CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
		return new CIntegraleDansFormule(listeCible, calculASommer.getClone(listeSource, listeCible), bornea.getClone(listeSource, listeCible),
				borneb.getClone(listeSource, listeCible), nomVariable, dansFormuleComplexe);
	}

	@Override
	public CCalculBase getCopie() {
		return new CIntegraleDansFormule(listeProprietaire, calculASommer.getCopie(), bornea.getCopie(),
				borneb.getCopie(), nomVariable, dansFormuleComplexe);
	}
	
  @Override
  public boolean dependDe(CElementBase p) {
  	return calculASommer.dependDe(p) || bornea.dependDe(p) || borneb.dependDe(p);
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	return calculASommer.dependDePourReclassement(p) || bornea.dependDePourReclassement(p)
  	|| borneb.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return calculASommer.dependDePourBoucle(p) || bornea.dependDePourBoucle(p)
  	|| borneb.dependDePourBoucle(p);
  }

  @Override
	public boolean existe(boolean infoRandom) {
  	return calculASommer.existe(infoRandom) && bornea.existe(infoRandom) && borneb.existe(infoRandom);
  }
  
  @Override
	public boolean estConstant() {
		return calculASommer.estConstant() && bornea.estConstant() && borneb.estConstant();
	}
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		calculASommer.estDefiniParObjetDans(listeOb)
		&& bornea.estDefiniParObjetDans(listeOb)
  	&& borneb.estDefiniParObjetDans(listeOb);
	}	     
 	
  @Override
  public double resultat(boolean infoRandom) throws ErreurCalculException {
  	listeProprietaire.nombreIterations += CCalculBase.nombreSubdivisionsPourSimpson;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	double[] val = new double[1];
  	double dbornea = bornea.resultat(infoRandom);
  	double dborneb = borneb.resultat(infoRandom);
  	return integrale(calculASommer, val, dbornea, dborneb, infoRandom);
  }
  
  @Override
  public void resultatComplexe(boolean infoRandom, Complexe zRes)  throws ErreurCalculException {
  	listeProprietaire.nombreIterations += CCalculBase.nombreSubdivisionsPourSimpson;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	double[] val = new double[1];
  	Complexe cbornea = new Complexe();
  	bornea.resultatComplexe(infoRandom, cbornea);
  	if (cbornea.y != 0) throw new ErreurCalculException();
  	double dbornea = cbornea.x;
  	Complexe cborneb = new Complexe();
  	borneb.resultatComplexe(infoRandom, cborneb);
  	if (cborneb.y != 0) throw new ErreurCalculException();
  	double dborneb = cborneb.x;
  	zRes.y = 0;
  	zRes.x = integrale(calculASommer, val, dbornea, dborneb, infoRandom);
  	return;  
  }

  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
  	listeProprietaire.nombreIterations += CCalculBase.nombreSubdivisionsPourSimpson;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	double[] val = new double[2];
  	val[0] = valeurParametre;
  	double dbornea = bornea.resultatFonction(infoRandom, valeurParametre);
  	double dborneb = borneb.resultatFonction(infoRandom, valeurParametre);
  	return integrale(calculASommer, val, dbornea, dborneb, infoRandom);
  }
	
  // calculASommer pointe sur une fonction de n+1 variables, n tant la  dmiension de valeurParamre
  // La dernire variable sert  appeler la fonction pour les diffrentes valeurs de l'indice.
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) throws ErreurCalculException {
  	listeProprietaire.nombreIterations += CCalculBase.nombreSubdivisionsPourSimpson;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	int n = valeurParametre.length;
  	double[] val = new double[n + 1];
  	for (int i = 0; i < n; i++) val[i] = valeurParametre[i];
  	double dbornea = bornea.resultatFonction(infoRandom, valeurParametre);
  	double dborneb = borneb.resultatFonction(infoRandom, valeurParametre);
  	return integrale(calculASommer, val, dbornea, dborneb, infoRandom);
  }
  
  @Override
	public String chaineCalcul(String[] varFor) {
  	String[] varFor2;
  	if (dansFormuleComplexe) {
  		// Dans le cas d'un calcul d'intgrale dans un calcul complexe, les paramtres ventuels complexes
  		// n'ont pas pu tre utiliss
  		varFor2 = new String[1];
  		varFor2[0] = nomVariable;
  	}
  	else {
	  	int n = 0;
	  	if (varFor == null) varFor2 = new String[1];
	  	else {
		  	n = varFor.length; 
		  	varFor2 = new String[n+1];
	  	}
	  	for (int i = 0; i < n; i++) varFor2[i] = varFor[i];
	  	varFor2[n] = nomVariable;
  	}
  	return getString("integrale") + "(" + calculASommer.chaineCalculSansPar(varFor2) + ","
		+ nomVariable + "," + bornea.chaineCalculSansPar(varFor) + ","
		+ borneb.chaineCalculSansPar(varFor) + ")";
  	
  }

  // Modifi version 4.7.7 pour mettre des parenthses si ncessaire
  @Override
	public String chaineLatex(String[] varFor) {
  	String[] varFor2;
  	String ch;
  	if (dansFormuleComplexe) {
  		// Dans le cas d'un calcul d'intgrale dans un calcul complexe, les paramtres ventuels complexes
  		// n'ont pas pu tree utiliss
  		varFor2 = new String[1];
  		varFor2[0] = nomVariable;
  	}
  	else {
	  	int n = 0;
	  	if (varFor == null) varFor2 = new String[1];
	  	else {
		  	n = varFor.length; 
		  	varFor2 = new String[n+1];
	  	}
	  	for (int i = 0; i < n; i++) varFor2[i] = varFor[i];
	  	varFor2[n] = nomVariable;
  	}
  	long nat = calculASommer.nature();
  	if (nat == CCalculBase.natOperation) {
  		COperation op = (COperation) calculASommer;
  		if ((op.operateur == Operateur.Plus) || (op.operateur == Operateur.Moin)) ch = calculASommer.chaineLatex(varFor2);
  		else ch = calculASommer.chaineLatexSansPar(varFor2);
  	}
  	else ch = calculASommer.chaineLatexSansPar(varFor2);
  	// String ch = calculASommer.chaineLatexSansPar(varFor2) + "d" + varFor2;
  	return "\\int_{" + bornea.chaineLatexSansPar(varFor) + "}" + "^{" + borneb.chaineLatexSansPar(varFor) + "}"
  		+ ch + "d" + varFor2[varFor2.length-1];
  }
   
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    nomVariable = inps.readUTF();
    calculASommer = (CCalculBase) (inps.readObject(list));
		bornea = (CCalculBase) (inps.readObject(list));
		borneb = (CCalculBase) (inps.readObject(list));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(nomVariable);
    oups.writeObject(calculASommer);
    oups.writeObject(bornea);
    oups.writeObject(borneb);
  }
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	return !calculASommer.dependDeVariable(indiceVariable) && !bornea.dependDeVariable(indiceVariable)
  	&& !borneb.dependDeVariable(indiceVariable);
  }

  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	return new CIntegraleDansFormule(listeProprietaire, calculASommer.calculAvecValeursRemplacees(),
  			bornea.calculAvecValeursRemplacees(), borneb.calculAvecValeursRemplacees(), nomVariable, dansFormuleComplexe) ;
  }
  
}
